# passerelle - uniform access to multiple data sources and services
# Copyright (C) 2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from contextlib import contextmanager
import datetime
import json
import mock
import os

import pytest
import xmlschema

from passerelle.contrib.toulouse_axel.models import (
    AxelError,
    Link,
    Lock,
    OperationResult,
    ToulouseAxel,
    form_maj_famille_dui,
    ref_famille_dui,
    ref_facture_a_payer,
    ref_facture_pdf,
    ref_verif_dui,
)
import utils


@pytest.fixture
def resource(db):
    return utils.make_resource(
        ToulouseAxel,
        slug='test',
        wsdl_url='http://example.net/AXEL_WS/AxelWS.php?wsdl')


@pytest.fixture
def link_params():
    return {
        'IDDUI': 'XXX',
        'PRENOM': 'John',
        'NOM': 'Doe',
        'NAISSANCE': '2010-10-10',
    }


@pytest.fixture
def update_params():
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/update_family_info.json')
    with open(filepath) as jsonfile:
        content = jsonfile.read()
    return json.loads(content)


@pytest.fixture
def flat_update_params():
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/flat_update_family_info.json')
    with open(filepath) as jsonfile:
        content = jsonfile.read()
    return json.loads(content)


def test_lock(app, resource):
    resp = app.get('/toulouse-axel/test/lock?key=&locker=', status=400)
    assert resp.json['err_desc'] == "key is empty"
    assert resp.json['err'] == 'bad-request'

    assert Lock.objects.count() == 0
    resp = app.get('/toulouse-axel/test/lock?key=foobar&locker=')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is True
    assert resp.json['locker'] == ''
    assert resp.json['lock_date'] is not None
    lock_date = resp.json['lock_date']
    lock = Lock.objects.latest('pk')
    assert lock.resource == resource
    assert lock.key == 'foobar'
    assert lock.locker == ''

    # again
    resp = app.get('/toulouse-axel/test/lock?key=foobar&locker=')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is True
    assert Lock.objects.count() == 1
    assert resp.json['locker'] == ''
    assert resp.json['lock_date'] == lock_date


def test_lock_with_locker(app, resource):
    assert Lock.objects.count() == 0
    resp = app.get('/toulouse-axel/test/lock?key=foobar&locker=something')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is True
    assert resp.json['locker'] == 'something'
    assert resp.json['lock_date'] is not None
    lock_date = resp.json['lock_date']
    lock = Lock.objects.latest('pk')
    assert lock.resource == resource
    assert lock.key == 'foobar'
    assert lock.locker == 'something'

    # again
    resp = app.get('/toulouse-axel/test/lock?key=foobar&locker=')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is True
    assert Lock.objects.count() == 1
    assert resp.json['locker'] == 'something'
    assert resp.json['lock_date'] == lock_date


def test_unlock(app, resource):
    Lock.objects.create(resource=resource, key='foobar', locker='something')
    resp = app.get('/toulouse-axel/test/unlock?key=foobar')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is False
    assert resp.json['locker'] == 'something'
    assert resp.json['lock_date'] is not None
    assert Lock.objects.count() == 0

    # again
    resp = app.get('/toulouse-axel/test/unlock?key=foobar')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is False
    assert 'locker' not in resp.json
    assert 'lock_date' not in resp.json
    assert Lock.objects.count() == 0


def test_locked(app, resource):
    assert Lock.objects.count() == 0
    resp = app.get('/toulouse-axel/test/locked?key=foobar')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is False
    assert 'locker' not in resp.json
    assert 'lock_date' not in resp.json

    Lock.objects.create(resource=resource, key='foobar', locker='something')
    resp = app.get('/toulouse-axel/test/locked?key=foobar')
    assert resp.json['err'] == 0
    assert resp.json['key'] == 'foobar'
    assert resp.json['locked'] is True
    assert resp.json['locker'] == 'something'
    assert resp.json['lock_date'] is not None


def test_operation_status_error(resource):
    resp = '''
    <?xml version="1.0"?>
    <PORTAILSERVICE>
      <RESULTAT>
        <TYPE>RefVerifDui</TYPE>
        <STATUS>NOK</STATUS>
        <COMMENTAIRES><![CDATA[Foo reason]]></COMMENTAIRES>
      </RESULTAT>
      <DATA>
        <PORTAIL/>
      </DATA>
    </PORTAILSERVICE>
    '''.strip()
    with mock.patch('passerelle.contrib.toulouse_axel.models.ToulouseAxel.soap_client') as client:
        client.return_value.service.getData.return_value = resp
        with pytest.raises(AxelError, match='Foo reason'):
            ref_verif_dui(resource, {
                'PORTAIL': {
                    'DUI': {
                        'IDDUI': 'XXX',
                        'IDPERSONNE': '42',
                        'PRENOM': 'John',
                        'NOM': 'Doe',
                        'NAISSANCE': '2010-10-10',
                    }
                }
            })


@contextmanager
def mock_getdata(content, operation):
    with mock.patch('passerelle.contrib.toulouse_axel.models.ToulouseAxel.soap_client') as client:
        resp = '''
        <?xml version="1.0"?>
        <PORTAILSERVICE>
          <RESULTAT>
            <TYPE>%s</TYPE>
            <STATUS>OK</STATUS>
            <DATE>10/10/2010 10:10:01</DATE>
            <COMMENTAIRES><![CDATA[]]></COMMENTAIRES>
          </RESULTAT>
          <DATA>
            %s
          </DATA>
        </PORTAILSERVICE>
        '''.strip() % (operation, content)
        client.return_value.service.getData.return_value = resp
        yield


@pytest.mark.parametrize('content', [
    '<PORTAIL><DUI/></PORTAIL>',
    '<PORTAIL><DUI><CODE>foo</CODE></DUI></PORTAIL>',
    '<PORTAIL><DUI><CODE>42</CODE></DUI></PORTAIL>',
])
def test_operation_ref_verif_dui(resource, content):
    with mock_getdata(content, 'RefVerifDui'):
        with pytest.raises(AxelError):
            ref_verif_dui(resource, {
                'PORTAIL': {
                    'DUI': {
                        'PRENOM': 'John',
                        'NOM': 'Doe',
                        'NAISSANCE': '2010-10-10',
                    }
                }
            })


@pytest.mark.parametrize('content', [
    '<PORTAIL><DUI/></PORTAIL>',
])
def test_operation_ref_famille_dui(resource, content):
    with mock_getdata(content, 'RefFamilleDui'):
        with pytest.raises(AxelError):
            ref_famille_dui(resource, {
                'PORTAIL': {
                    'DUI': {
                        'IDDUI': 'XXX',
                    }
                }
            })


@pytest.mark.parametrize('content', [
    '<PORTAIL><DUI/></PORTAIL>',
])
def test_operation_form_maj_famille_dui(resource, content):
    with mock_getdata(content, 'FormMajFamilleDui'):
        with pytest.raises(AxelError):
            form_maj_famille_dui(resource, {
                'PORTAIL': {
                    'DUI': {
                        'IDDUI': 'XXX',
                    }
                }
            })


@pytest.mark.parametrize('content', [
    '<PORTAIL><DUI/></PORTAIL>',
])
def test_operation_ref_facture_a_payer(resource, content):
    with mock_getdata(content, 'RefFactureAPayer'):
        with pytest.raises(AxelError):
            ref_facture_a_payer(resource, {
                'PORTAIL': {
                    'DUI': {
                        'IDDUI': 'XXX',
                    }
                }
            })


@pytest.mark.parametrize('content', [
    "<PORTAIL><PDF FOO='BAR'></PDF></PORTAIL>",
])
def test_operation_ref_facture_pdf(resource, content):
    with mock_getdata(content, 'RefFacturePDF'):
        with pytest.raises(AxelError):
            ref_facture_pdf(resource, {
                'PORTAIL': {
                    'FACTUREPDF': {
                        'IDFACTURE': 42,
                    }
                }
            })


def test_link_endpoint_nameid_empty(app, resource, link_params):
    resp = app.post_json('/toulouse-axel/test/link?NameID=', params=link_params, status=400)
    assert resp.json['err_desc'] == "NameID is empty"
    assert resp.json['err'] == 'bad-request'


def test_link_endpoint_axel_error(app, resource, link_params):
    with mock.patch('passerelle.contrib.toulouse_axel.models.ref_verif_dui') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'
    assert resp.json['data'] == {'xml_request': None, 'xml_response': None}

    # test xml_request and xml_response only for this endpoint
    xml_request = """<PORTAIL>
  <DUI>
    <IDDUI>XXX</IDDUI>
    <IDPERSONNE />
    <NOM>Doe</NOM>
    <PRENOM>John</PRENOM>
    <NAISSANCE>10/10/2010</NAISSANCE>
  </DUI>
</PORTAIL>
"""

    with mock_getdata('', 'RefVerifDui'):
        with mock.patch('xmlschema.XMLSchema.validate') as xml_validate:
            xml_validate.side_effect = xmlschema.XMLSchemaValidationError(None, None)
            resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'].startswith("Axel error: invalid request")
    assert resp.json['err'] == 'error'
    assert resp.json['data']['xml_request'] == xml_request
    assert resp.json['data']['xml_response'] is None

    xml_response = """<PORTAILSERVICE>
  <RESULTAT>
    <TYPE>RefVerifDui</TYPE>
    <STATUS>NOK</STATUS>
    <COMMENTAIRES>Foo reason</COMMENTAIRES>
  </RESULTAT>
  <DATA>
    <PORTAIL />
  </DATA>
</PORTAILSERVICE>"""
    response = """
    <?xml version="1.0"?>
    <PORTAILSERVICE>
      <RESULTAT>
        <TYPE>RefVerifDui</TYPE>
        <STATUS>NOK</STATUS>
        <COMMENTAIRES><![CDATA[Foo reason]]></COMMENTAIRES>
      </RESULTAT>
      <DATA>
        <PORTAIL/>
      </DATA>
    </PORTAILSERVICE>
    """.strip()
    with mock.patch('passerelle.contrib.toulouse_axel.models.ToulouseAxel.soap_client') as client:
        client.return_value.service.getData.return_value = response
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'] == 'Axel error: Foo reason'
    assert resp.json['err'] == 'error'
    assert resp.json['data']['xml_request'] == xml_request
    assert resp.json['data']['xml_response'] == xml_response

    content = """<PORTAIL>
      <DUI>
        <IDDUI>XXX</IDDUI>
        <IDPERSONNE>42</IDPERSONNE>
        <CODE>2</CODE>
      </DUI>
    </PORTAIL>"""
    xml_response = """<PORTAILSERVICE>
  <RESULTAT>
    <TYPE>RefVerifDui</TYPE>
    <STATUS>OK</STATUS>
    <DATE>10/10/2010 10:10:01</DATE>
    <COMMENTAIRES />
  </RESULTAT>
  <DATA>
    %s
  </DATA>
</PORTAILSERVICE>""" % content
    with mock_getdata(content, 'RefVerifDui'):
        with mock.patch('passerelle.contrib.toulouse_axel.models.AxelSchema.decode') as decode:
            decode.side_effect = xmlschema.XMLSchemaValidationError(None, None)
            resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'].startswith("Axel error: invalid response")
    assert resp.json['err'] == 'error'
    assert resp.json['data']['xml_request'] == xml_request
    assert resp.json['data']['xml_response'] == xml_response


@pytest.mark.parametrize('xml_response', [
    '<IDDUI>XXX</IDDUI><IDPERSONNE/><CODE>0</CODE>',
    '<IDDUI>XXX</IDDUI><IDPERSONNE>42</IDPERSONNE><CODE>1</CODE>',
    '<IDDUI>XXX</IDDUI><IDPERSONNE>42</IDPERSONNE><CODE>4</CODE>',
])
def test_link_endpoint_no_result(app, resource, link_params, xml_response):
    content = '''<PORTAIL>
    <DUI>
        %s
    </DUI>
</PORTAIL>''' % xml_response
    with mock_getdata(content, 'RefVerifDui'):
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'


def test_link_endpoint_conflict(app, resource, link_params):
    content = '''<PORTAIL>
    <DUI>
        <IDDUI>XXX</IDDUI>
        <IDPERSONNE>42</IDPERSONNE>
        <CODE>2</CODE>
    </DUI>
</PORTAIL>'''
    # existing link but dui is wrong
    link = Link.objects.create(resource=resource, name_id='yyy', dui='YYY', person_id='42')
    with mock_getdata(content, 'RefVerifDui'):
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'] == "Data conflict"
    assert resp.json['err'] == 'conflict'

    # existing link but person_id is wrong
    link.dui = 'XXX'
    link.person_id = '35'
    link.save()
    with mock_getdata(content, 'RefVerifDui'):
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert resp.json['err_desc'] == "Data conflict"
    assert resp.json['err'] == 'conflict'


@pytest.mark.parametrize('code', [2, 3])
def test_link_endpoint(app, resource, link_params, code):
    content = '''<PORTAIL>
    <DUI>
        <IDDUI>XXX</IDDUI>
        <IDPERSONNE>42</IDPERSONNE>
        <CODE>%s</CODE>
    </DUI>
</PORTAIL>''' % code
    with mock_getdata(content, 'RefVerifDui'):
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert set(resp.json.keys()) == set(['err', 'link', 'created', 'dui', 'data'])
    assert resp.json['err'] == 0
    assert resp.json['dui'] == 'XXX'
    assert resp.json['created'] is True
    assert 'xml_request' in resp.json['data']
    assert 'xml_response' in resp.json['data']

    # again
    with mock_getdata(content, 'RefVerifDui'):
        resp = app.post_json('/toulouse-axel/test/link?NameID=yyy', params=link_params)
    assert set(resp.json.keys()) == set(['err', 'link', 'created', 'dui', 'data'])
    assert resp.json['err'] == 0
    assert resp.json['dui'] == 'XXX'
    assert resp.json['created'] is False  # link already exists
    assert 'xml_request' in resp.json['data']
    assert 'xml_response' in resp.json['data']


def test_unlink_endpoint_no_result(app, resource):
    resp = app.post('/toulouse-axel/test/unlink?NameID=yyy')
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'


def test_unlink_endpoint(app, resource):
    link = Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    resp = app.post('/toulouse-axel/test/unlink?NameID=yyy')
    assert Link.objects.exists() is False
    assert resp.json['err'] == 0
    assert resp.json['link'] == link.pk
    assert resp.json['dui'] == 'XXX'
    assert resp.json['deleted'] is True


def test_family_info_endpoint_axel_error(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    with mock.patch('passerelle.contrib.toulouse_axel.models.ref_famille_dui') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.get('/toulouse-axel/test/family_info?NameID=yyy')
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'


def test_family_info_endpoint_no_result(app, resource):
    resp = app.get('/toulouse-axel/test/family_info?NameID=yyy')
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'


def test_family_info_endpoint(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/family_info.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFamilleDui'):
        resp = app.get('/toulouse-axel/test/family_info?NameID=yyy')
    assert resp.json['err'] == 0
    assert set(resp.json['data'].keys()) == set([
        'ADRESSE',
        'CODEMISEAJOUR',
        'DEMATFACTURES',
        'ENFANT',
        'IDDUI',
        'NBENFANTACTIF',
        'NBRLACTIF',
        'REACTUALISATIONENLIGNE',
        'REVENUS',
        'RL1',
        'RL2',
        'SITUATIONFAMILIALE',
        'TELFIXE',
    ])


def test_child_info_endpoint_axel_error(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    with mock.patch('passerelle.contrib.toulouse_axel.models.ref_famille_dui') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.get('/toulouse-axel/test/child_info?NameID=yyy&idpersonne=zzz')
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'


def test_child_info_endpoint_no_result(app, resource):
    resp = app.get('/toulouse-axel/test/child_info?NameID=yyy&idpersonne=zzz')
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'

    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/family_info.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFamilleDui'):
        resp = app.get('/toulouse-axel/test/child_info?NameID=yyy&idpersonne=zzz')
    assert resp.json['err_desc'] == "Child not found"
    assert resp.json['err'] == 'not-found'


def test_child_info_endpoint(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/family_info.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFamilleDui'):
        resp = app.get('/toulouse-axel/test/child_info?NameID=yyy&idpersonne=4242')
    assert resp.json['err'] == 0
    assert set(resp.json['data'].keys()) == set([
        'ASSURANCE',
        'CONTACT',
        'DATENAISSANCE',
        'IDPERSONNE',
        'PRENOM',
        'PRENOMMERE',
        'PRENOMPERE',
        'NOM',
        'NOMMERE',
        'NOMPERE',
        'RATTACHEAUTREDUI',
        'SANITAIRE',
        'SEXE',
    ])


def test_update_family_info_endpoint_axel_error(app, resource, update_params):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    with mock.patch('passerelle.contrib.toulouse_axel.models.form_maj_famille_dui') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.post_json('/toulouse-axel/test/update_family_info?NameID=yyy', params=update_params)
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'
    assert 'error_post_data' in resp.json['data']


def test_update_family_info_endpoint_no_result(app, resource, update_params):
    resp = app.post_json('/toulouse-axel/test/update_family_info?NameID=yyy', params=update_params)
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'


def test_update_family_info_endpoint(app, resource, update_params):
    link = Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    content = "<PORTAIL/>"
    with mock_getdata(content, 'FormMajFamilleDui'):
        resp = app.post_json('/toulouse-axel/test/update_family_info?NameID=yyy', params=update_params)
    assert resp.json['err'] == 0
    assert resp.json['dui'] == 'XXX'
    assert resp.json['updated'] is True
    assert 'data' in resp.json
    assert 'xml_request' in resp.json['data']
    assert 'xml_response' in resp.json['data']

    with mock.patch('passerelle.contrib.toulouse_axel.models.form_maj_famille_dui') as operation:
        operation.return_value = OperationResult(json_response={}, xml_request='', xml_response='')
        resp = app.post_json('/toulouse-axel/test/update_family_info?NameID=yyy', params=update_params)
    assert operation.call_args_list[0][0][1]['PORTAIL']['DUI']['IDDUI'] == 'XXX'
    assert operation.call_args_list[0][0][1]['PORTAIL']['DUI']['DATEDEMANDE'] == datetime.date.today().strftime('%Y-%m-%d')
    assert operation.call_args_list[0][0][1]['PORTAIL']['DUI']['QUIACTUALISEDUI'] == '1'

    link.person_id = '35'
    link.save()
    with mock.patch('passerelle.contrib.toulouse_axel.models.form_maj_famille_dui') as operation:
        operation.return_value = OperationResult(json_response={}, xml_request='', xml_response='')
        resp = app.post_json('/toulouse-axel/test/update_family_info?NameID=yyy', params=update_params)
    assert operation.call_args_list[0][0][1]['PORTAIL']['DUI']['IDDUI'] == 'XXX'
    assert operation.call_args_list[0][0][1]['PORTAIL']['DUI']['DATEDEMANDE'] == datetime.date.today().strftime('%Y-%m-%d')
    assert operation.call_args_list[0][0][1]['PORTAIL']['DUI']['QUIACTUALISEDUI'] == '2'


def test_update_family_info_flat_endpoint(app, resource, flat_update_params):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    content = "<PORTAIL/>"
    with mock_getdata(content, 'FormMajFamilleDui'):
        resp = app.post_json('/toulouse-axel/test/update_family_info?NameID=yyy', params=flat_update_params)
    assert resp.json['err'] == 0
    assert resp.json['dui'] == 'XXX'
    assert resp.json['updated'] is True
    assert 'data' in resp.json
    assert 'xml_request' in resp.json['data']
    assert 'xml_response' in resp.json['data']


def test_invoices_endpoint_axel_error(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    with mock.patch('passerelle.contrib.toulouse_axel.models.ref_facture_a_payer') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.get('/toulouse-axel/test/invoices?NameID=yyy')
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'


def test_invoices_endpoint_no_result(app, resource):
    resp = app.get('/toulouse-axel/test/invoices?NameID=yyy')
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'


def test_invoices_endpoint_no_invoices(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    content = '''<PORTAIL>
    <DUI>
        <IDDUI>XXX</IDDUI>
        <CHOIXDEMAT>0</CHOIXDEMAT>
        <NBFACTURES>0</NBFACTURES>
    </DUI>
</PORTAIL>'''
    with mock_getdata(content, 'RefFactureAPayer'):
        resp = app.get('/toulouse-axel/test/invoices?NameID=yyy')
    assert resp.json['err'] == 0
    assert resp.json['data'] == []


def test_invoices_endpoint(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/invoices.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFactureAPayer'):
        resp = app.get('/toulouse-axel/test/invoices?NameID=yyy')
    assert resp.json['err'] == 0
    assert resp.json['data'] == [
        {
            'id': 42,
            'label': 'PRESTATIONS PERISCOLAIRES SEPTEMBRE-OCTOBRE 2019',
            'amount': '44.94',
            'total_amount': '44.94',
            'online_payment': True,
            'created': '2019-11-12',
            'pay_limit_date': '2019-12-04',
            'has_pdf': True,
            'paid': False,
            'vendor': {
                'toulouse-axel': {
                    'IDFACTURATION': '4242-35AA',
                    'IDFACTURE': 42,
                    'IDREGIE': 'MAREGIE',
                    'DATEECHEANCE': '2019-12-04',
                    'DATEEMISSION': '2019-11-12',
                    'EXISTEPDF': '1',
                    'LIBELLE': 'PRESTATIONS PERISCOLAIRES SEPTEMBRE-OCTOBRE 2019',
                    'MONTANTTOTAL': '44.94',
                    'NUMFACTURE': 42,
                    'RESTEAPAYER': '44.94',
                 }
            }
        },
        {
            'id': 43,
            'label': 'PRESTATIONS PERISCOLAIRES NOVEMBRE 2019',
            'amount': '44.94',
            'total_amount': '44.94',
            'online_payment': True,
            'created': '2019-12-12',
            'pay_limit_date': '2020-01-04',
            'has_pdf': False,
            'paid': False,
            'vendor': {
                'toulouse-axel': {
                    'IDFACTURATION': '4243-35AA',
                    'IDFACTURE': 43,
                    'DATEECHEANCE': '2020-01-04',
                    'DATEEMISSION': '2019-12-12',
                    'EXISTEPDF': '0',
                    'IDREGIE': 'MAREGIE',
                    'LIBELLE': 'PRESTATIONS PERISCOLAIRES NOVEMBRE 2019',
                    'MONTANTTOTAL': '44.94',
                    'NUMFACTURE': 43,
                    'RESTEAPAYER': '44.94',
                }
            }
        }
    ]


def test_invoice_endpoint_axel_error(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    with mock.patch('passerelle.contrib.toulouse_axel.models.ref_facture_a_payer') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.get('/toulouse-axel/test/invoice/42?NameID=yyy')
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'


def test_invoice_endpoint_no_result(app, resource):
    resp = app.get('/toulouse-axel/test/invoice/42?NameID=yyy')
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'

    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/invoices.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFactureAPayer'):
        resp = app.get('/toulouse-axel/test/invoice/35?NameID=yyy')
    assert resp.json['err_desc'] == "Invoice not found"
    assert resp.json['err'] == 'not-found'


def test_invoice_endpoint(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/invoices.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFactureAPayer'):
        resp = app.get('/toulouse-axel/test/invoice/42?NameID=yyy')
    assert resp.json['err'] == 0
    assert resp.json['data'] == {
        'id': 42,
        'label': 'PRESTATIONS PERISCOLAIRES SEPTEMBRE-OCTOBRE 2019',
        'amount': '44.94',
        'total_amount': '44.94',
        'online_payment': True,
        'created': '2019-11-12',
        'pay_limit_date': '2019-12-04',
        'has_pdf': True,
        'paid': False,
        'vendor': {
            'toulouse-axel': {
                'IDFACTURATION': '4242-35AA',
                'IDFACTURE': 42,
                'IDREGIE': 'MAREGIE',
                'DATEECHEANCE': '2019-12-04',
                'DATEEMISSION': '2019-11-12',
                'EXISTEPDF': '1',
                'LIBELLE': 'PRESTATIONS PERISCOLAIRES SEPTEMBRE-OCTOBRE 2019',
                'MONTANTTOTAL': '44.94',
                'NUMFACTURE': 42,
                'RESTEAPAYER': '44.94',
            }
        }
    }


def test_invoice_pdf_endpoint_axel_error(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    with mock.patch('passerelle.contrib.toulouse_axel.models.ref_facture_a_payer') as operation:
        operation.side_effect = AxelError('FooBar')
        resp = app.get('/toulouse-axel/test/invoice/42/pdf?NameID=yyy', status=404)
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'

    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/invoices.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFactureAPayer'):
        with mock.patch('passerelle.contrib.toulouse_axel.models.ref_facture_pdf') as operation:
            operation.side_effect = AxelError('FooBar')
            resp = app.get('/toulouse-axel/test/invoice/42/pdf?NameID=yyy', status=404)
    assert resp.json['err_desc'] == "Axel error: FooBar"
    assert resp.json['err'] == 'error'


def test_invoice_pdf_endpoint_no_result(app, resource):
    resp = app.get('/toulouse-axel/test/invoice/42/pdf?NameID=yyy', status=404)
    assert resp.json['err_desc'] == "Person not found"
    assert resp.json['err'] == 'not-found'

    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    filepath = os.path.join(os.path.dirname(__file__), 'data/toulouse_axel/invoices.xml')
    with open(filepath) as xml:
        content = xml.read()
    with mock_getdata(content, 'RefFactureAPayer'):
        resp = app.get('/toulouse-axel/test/invoice/35/pdf?NameID=yyy', status=404)
    assert resp.json['err_desc'] == "Invoice not found"
    assert resp.json['err'] == 'not-found'

    with mock_getdata(content, 'RefFactureAPayer'):
        resp = app.get('/toulouse-axel/test/invoice/43/pdf?NameID=yyy', status=404)
    assert resp.json['err_desc'] == "PDF not available"
    assert resp.json['err'] == 'not-available'

    pdf_content = '''<PORTAIL>
    <PDF FILE=''></PDF>
</PORTAIL>'''
    with mock.patch('passerelle.contrib.toulouse_axel.models.ToulouseAxel.get_invoice') as invoice:
        invoice.return_value = {'has_pdf': True}
        with mock_getdata(pdf_content, 'RefFacturePDF'):
            resp = app.get('/toulouse-axel/test/invoice/42/pdf?NameID=yyy', status=404)
    assert resp.json['err_desc'] == "PDF error"
    assert resp.json['err'] == 'error'


def test_invoice_pdf_endpoint(app, resource):
    Link.objects.create(resource=resource, name_id='yyy', dui='XXX', person_id='42')
    pdf_content = '''<PORTAIL>
    <PDF FILE='aGVsbG8gd29ybGQ='></PDF>
</PORTAIL>'''
    with mock.patch('passerelle.contrib.toulouse_axel.models.ToulouseAxel.get_invoice') as invoice:
        invoice.return_value = {'has_pdf': True}
        with mock_getdata(pdf_content, 'RefFacturePDF'):
            app.get('/toulouse-axel/test/invoice/42/pdf?NameID=yyy')
